/**
* A class that represents a rational number. 
*/

public class Rational {
  
  // instance variables
  private int numerator;
  private int denominator;
  
  /**
  * The default constructor for objects of class Rational.  
  * Creates the rational number 1.
  */
  public Rational() { 
    this(1, 1);
    //numerator = 1;
    //denominator = 1;
  }
  
  public Rational(int n) { this(n, 1); } 
  
  /**
  * The alternate constructor for objects of class Rational.  
  * Creates a rational number equivalent to n/d.
  */    
  public Rational(int n, int d) {
    numerator = n;
    denominator = d;
    if(denominator == 0)
      throw new ZeroDenominatorException(
        "Can not have a zero denominator");
    normalize();
  }
  
  /**
  * Get the value of the Numerator
  * @return     the value of the numerator
  */
  public int getNumerator() {
    //dummy return 0;
    return numerator;
  }
  
  /**
  * Get the value of the Denominator
  * @return     the value of the denominator
  */
  public int getDenominator() {
    return denominator;
  }
  
  /**
  * Negate a rational number r
  * @return a new rational number that is negation of this number -r
  */    
  public Rational negate() {               
    return new Rational(-1*numerator, denominator);
  }
  
  /**
  * Invert a rational number r 
  * @return a new rational number that is 1/r.
  */    
  public Rational invert() {               
    //dummy return null;
    return new Rational(denominator, numerator);
  }
  
  /**
  * Add two rational numbers
  * @param other the second argument of the add
  * @return a new rational number that is the sum of this and the 
  * other rational
  */    
  public Rational add(Rational other) {       

    int resultDenominator = denominator*other.denominator;
    int resultNumerator = numerator*other.denominator 
                        + other.numerator*denominator;
    
    return new Rational(resultNumerator, resultDenominator);
  }
  
  /**
  * Subtract a rational number t from this one r
  * @param other the second argument of subtract 
  * @return a new rational number that is r-t
  */    
  public Rational subtract(Rational other) {               
    return add(other.negate());
  }
  
  /**
  * Multiply two rational numbers
  * @param other the second argument of multiply
  * @return a new rational number that is the sum of this 
  * object and the other rational.
  */    
  public Rational multiply(Rational other) {       
    int resultDenominator = denominator*other.denominator;
    int resultNumerator = numerator*other.numerator ;
    return new Rational(resultNumerator, resultDenominator);
  }
  
  
  /**
  * Divide a this rational number r by another one t
  * @param other the second argument of divide 
  * @return a new rational number that is r/t
  */    
  public Rational divide(Rational other) {               
    return multiply(other.invert());
  }
  
  public boolean equals(Object other) {
    
    if (other instanceof Rational) {
      Rational o = (Rational)other;
      
      return (numerator == o.numerator &&
              denominator == o.denominator);
    }
    return false;
    
  }
  
  
  /**
  * Put the rational number in normal form where the numerator
  * and the denominator share no common factors.  
  * Guarantee that only the numerator is negative.
  */
  private void normalize() {
    int absNumerator = Math.abs(numerator);
    int absDenominator = Math.abs(denominator);
    int signDenominator = denominator/absDenominator;
    
    int divideBy = gcd(absNumerator, absDenominator);
    numerator = (numerator / divideBy) * signDenominator;
    denominator = (denominator / divideBy) * signDenominator;
  }
  
  /**
  * Recursively compute the greatest common divisor of 
  * two positive integers
  * @param a the first argument of gcd
  * @param b the second argument of gcd
  * @return the gcd of the two arguments
  */
  public static int gcd(int a, int b) {
    if (a<b)
      return gcd(b, a);
    else if(b==0)
      return a;
    
    int remainder = a % b;
    return gcd(b, remainder);
  }
  
  public static Rational add(Rational other1, Rational other2) {       

    int resultDenominator = other1.denominator*other2.denominator;
    int resultNumerator = other1.numerator*other2.denominator 
                        + other2.numerator*other1.denominator;
    
    return new Rational(resultNumerator, resultDenominator);
  }
    
  
  public String toString() {
    return "(" + this.numerator + "/" + this.denominator + ")";
  }

  
}

class ZeroDenominatorException extends RuntimeException { 
  
  public ZeroDenominatorException(String mes) { 
    super(mes); 
  }
}
